"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAudio = void 0;
var react_1 = require("react");
var noop_1 = __importDefault(require("./shared/noop"));
var isClient_1 = __importDefault(require("./shared/isClient"));
var useObjectState_1 = __importDefault(require("./useObjectState"));
var isDevelopment_1 = __importDefault(require("./shared/isDevelopment"));
var isAPISupported_1 = __importDefault(require("./shared/isAPISupported"));
var createHandlerSetter_1 = __importDefault(require("./factory/createHandlerSetter"));
var warnOnce_1 = __importDefault(require("./shared/warnOnce"));
var defaultOptions = {
    volume: 1,
    loop: false,
    muted: false,
    playbackRate: 1,
    autoPlay: false,
    preload: 'auto',
};
var defaultState = __assign({ duration: 0, currentTime: 0, isPlaying: false, isSrcLoading: undefined }, defaultOptions);
var errorEventCodeToMessageMapper = {
    3: 'MEDIA_ERR_DECODE - error occurred when decoding',
    4: 'MEDIA_ERR_SRC_NOT_SUPPORTED - audio not supported',
    2: 'MEDIA_ERR_NETWORK - error occurred when downloading',
    1: 'MEDIA_ERR_ABORTED - fetching process aborted by user',
};
var hookNotSupportedControls = Object.freeze({
    seek: noop_1.default,
    play: noop_1.default,
    mute: noop_1.default,
    pause: noop_1.default,
    unmute: noop_1.default,
    onError: noop_1.default,
    setVolume: noop_1.default,
});
var checkIfRefElementExists = function (ref, callback) { return function () {
    var element = ref.current;
    if (!element) {
        return undefined;
    }
    return callback(element);
}; };
var useAudio = function (src, options) {
    var hookNotSupportedResponse = [
        defaultState,
        hookNotSupportedControls,
        (0, react_1.useRef)(null),
    ];
    if (!isClient_1.default) {
        if (!isDevelopment_1.default) {
            (0, warnOnce_1.default)('Please be aware that useAudio hook could not be available during SSR');
        }
        return hookNotSupportedResponse;
    }
    if (!(0, isAPISupported_1.default)('Audio')) {
        (0, warnOnce_1.default)('The current device does not support the \'Audio\' API, you should avoid using useAudio hook');
        return hookNotSupportedResponse;
    }
    var audioRef = (0, react_1.useRef)(new Audio(src));
    var _a = (0, createHandlerSetter_1.default)(), onErrorRef = _a[0], setOnErrorRef = _a[1];
    var _b = (0, useObjectState_1.default)(defaultState), state = _b[0], setState = _b[1];
    var onError = function (error) {
        if (onErrorRef.current) {
            onErrorRef.current(error);
        }
    };
    var play = (0, react_1.useCallback)(checkIfRefElementExists(audioRef, function (element) { return element
        .play()
        .then(function () {
        setState({
            isPlaying: true,
        });
    })
        .catch(onError); }), []);
    var pause = (0, react_1.useCallback)(checkIfRefElementExists(audioRef, function (element) {
        element.pause();
        setState({
            isPlaying: false,
        });
    }), []);
    var mute = (0, react_1.useCallback)(checkIfRefElementExists(audioRef, function (element) {
        // eslint-disable-next-line no-param-reassign
        element.muted = true;
        setState({
            muted: true,
        });
    }), []);
    var unmute = (0, react_1.useCallback)(checkIfRefElementExists(audioRef, function (element) {
        // eslint-disable-next-line no-param-reassign
        element.muted = false;
        setState({
            muted: false,
        });
    }), []);
    var seek = (0, react_1.useCallback)(function (time) { return checkIfRefElementExists(audioRef, function (element) {
        var newTime = time >= 0 ? Math.min(time, element.duration) : Math.max(time, 0);
        // eslint-disable-next-line no-param-reassign
        element.currentTime = newTime;
        setState({
            currentTime: newTime,
        });
    })(); }, []);
    var setVolume = (0, react_1.useCallback)(function (volume) { return checkIfRefElementExists(audioRef, function (element) {
        var newVolume = volume >= 0 ? Math.min(volume, 1) : Math.max(volume, 0);
        // eslint-disable-next-line no-param-reassign
        element.volume = newVolume;
        setState({
            volume: newVolume,
        });
    })(); }, []);
    var onLoadedData = checkIfRefElementExists(audioRef, function (element) { return setState({
        isSrcLoading: false,
        duration: element.duration,
        currentTime: element.currentTime,
    }); });
    var onTimeUpdate = checkIfRefElementExists(audioRef, function (element) { return setState({
        currentTime: element.currentTime,
    }); });
    var errorEventCallback = function () {
        var element = audioRef.current;
        var errorCode = element.error.code;
        var errorMessage = element.error.message
            || errorEventCodeToMessageMapper[errorCode]
            || 'UNKNOWN';
        onError(new Error(errorMessage));
    };
    (0, react_1.useEffect)(function () {
        var element = audioRef.current;
        if (element) {
            var mergedOptions = __assign(__assign({}, defaultOptions), options);
            element.loop = mergedOptions.loop;
            element.muted = mergedOptions.muted;
            element.volume = mergedOptions.volume;
            element.preload = mergedOptions.preload;
            element.autoplay = mergedOptions.autoPlay;
            element.playbackRate = mergedOptions.playbackRate;
            setState(__assign(__assign({}, mergedOptions), { isSrcLoading: true }));
            element.addEventListener('loadeddata', onLoadedData);
            element.addEventListener('timeupdate', onTimeUpdate);
            element.addEventListener('error', errorEventCallback);
        }
        return function () {
            if (element) {
                element.removeEventListener('loadeddata', onLoadedData);
                element.removeEventListener('timeupdate', onTimeUpdate);
                element.removeEventListener('error', errorEventCallback);
            }
            pause();
        };
    }, []);
    (0, react_1.useEffect)(function () {
        if (state.isSrcLoading === false && state.autoPlay) {
            play();
        }
    }, [state.isSrcLoading, state.autoPlay]);
    var controls = Object.freeze({
        seek: seek,
        play: play,
        mute: mute,
        pause: pause,
        unmute: unmute,
        setVolume: setVolume,
        onError: setOnErrorRef,
    });
    return [state, controls, audioRef];
};
exports.useAudio = useAudio;
exports.default = exports.useAudio;
